from io import BytesIO
from pathlib import Path
from functools import partial

from nonebot.adapters import Bot as BaseBot

from ..types import Text, Image, Reply, Mention
from ..utils import SupportedAdapters, register_ms_adapter

try:
    from nonebot.adapters.onebot.v12 import Bot, MessageSegment

    adapter = SupportedAdapters.onebot_v12
    register_onebot_v12 = partial(register_ms_adapter, adapter)

    @register_onebot_v12(Text)
    def _text(t: Text) -> MessageSegment:
        return MessageSegment.text(t.data["text"])

    @register_onebot_v12(Image)
    async def _image(i: Image, bot: BaseBot) -> MessageSegment:
        if not isinstance(bot, Bot):
            raise TypeError(f"Unsupported type of bot: {type(bot)}")
        image = i.data["image"]
        name = i.data["name"]
        if isinstance(image, str):
            resp = await bot.upload_file(type="url", name=name, url=image)
        elif isinstance(image, Path):
            resp = await bot.upload_file(
                type="path", name=name, path=str(image.resolve())
            )
        elif isinstance(image, BytesIO):
            image = image.getvalue()
            resp = await bot.upload_file(type="data", name=name, data=image)
        elif isinstance(image, bytes):
            resp = await bot.upload_file(type="data", name=name, data=image)
        else:
            raise TypeError(f"Unsupported type of image: {type(image)}")

        file_id = resp["file_id"]
        return MessageSegment.image(file_id)

    @register_onebot_v12(Mention)
    async def _mention(m: Mention) -> MessageSegment:
        return MessageSegment.mention(m.data["user_id"])

    @register_onebot_v12(Reply)
    async def _reply(r: Reply) -> MessageSegment:
        return MessageSegment.reply(r.data["message_id"])

except ImportError:
    pass
except Exception as e:
    raise e



from .helpers import register_ms_adapter
from .const import SupportedAdapters, supported_adapter_names
from .types import (
    MessageFactory,
    AdapterNotInstalled,
    AdapterNotSupported,
    MessageSegmentFactory,
)

__all__ = [
    "SupportedAdapters",
    "supported_adapter_names",
    "AdapterNotInstalled",
    "AdapterNotSupported",
    "MessageSegmentFactory",
    "MessageFactory",
    "register_ms_adapter",
]



from collections.abc import Awaitable
from typing import Type, TypeVar, Callable

from nonebot.internal.adapter.bot import Bot
from nonebot.internal.adapter.message import MessageSegment

from .const import SupportedAdapters
from .types import MessageSegmentFactory

T = TypeVar("T", bound=MessageSegmentFactory)
BuildFunc = (
    Callable[[T], MessageSegment | Awaitable[MessageSegment]]
    | Callable[[T, Bot], MessageSegment | Awaitable[MessageSegment]]
)


def register_ms_adapter(
    adapter: SupportedAdapters, ms_factory: Type[T]
) -> Callable[[BuildFunc], BuildFunc]:
    def decorator(builder: BuildFunc) -> BuildFunc:
        ms_factory._builders[adapter] = builder
        return builder

    return decorator



import asyncio
from abc import ABC
from copy import deepcopy
from inspect import signature
from typing_extensions import Self
from typing import TypeVar, Callable, Iterable, Awaitable, cast

from nonebot.internal.adapter.bot import Bot
from nonebot.internal.adapter.message import Message, MessageSegment

from .const import SupportedAdapters, supported_adapter_names


class AdapterNotInstalled(Exception):
    def __init__(self, adapter_name: str) -> None:
        message = f'adapter "{adapter_name}" not installed, please install first'
        super().__init__(self, message)


class AdapterNotSupported(Exception):
    def __init__(self, adapter_name: str) -> None:
        message = f'adapter "{adapter_name}" not supported'
        super().__init__(self, message)


TMSF = TypeVar("TMSF", bound="MessageSegmentFactory")
TMF = TypeVar("TMF", bound="MessageFactory")


class MessageSegmentFactory(ABC):
    _builders: dict[
        SupportedAdapters,
        Callable[[Self], MessageSegment | Awaitable[MessageSegment]]
        | Callable[[Self, Bot], MessageSegment | Awaitable[MessageSegment]],
    ]

    data: dict

    def __init_subclass__(cls) -> None:
        cls._builders = {}
        return super().__init_subclass__()

    def __eq__(self, other: Self) -> bool:
        return self.data == other.data

    async def build(self, bot: Bot) -> MessageSegment:
        adapter_name = bot.adapter.get_name()
        if adapter_name not in supported_adapter_names:
            raise AdapterNotSupported(adapter_name)
        if builder := self._builders[adapter_name]:
            if len(signature(builder).parameters) == 1:
                builder = cast(
                    Callable[[Self], MessageSegment | Awaitable[MessageSegment]],
                    builder,
                )
                res = builder(self)
            elif len(signature(builder).parameters) == 2:
                builder = cast(
                    Callable[[Self, Bot], MessageSegment | Awaitable[MessageSegment]],
                    builder,
                )
                res = builder(self, bot)
            else:
                raise RuntimeError()
            if asyncio.iscoroutine(res):
                return await res
            else:
                res = cast(MessageSegment, res)
                return res
        raise AdapterNotInstalled(adapter_name)

    def __add__(self: TMSF, other: str | TMSF | Iterable[TMSF]):
        return MessageFactory(self) + other

    def __radd__(self: TMSF, other: str | TMSF | Iterable[TMSF]):
        return MessageFactory(other) + self


class MessageFactory(list[TMSF]):
    _text_factory: Callable[[str], TMSF]
    _message_registry: dict[SupportedAdapters, type[Message]] = {}

    @classmethod
    def register_text_ms(cls, factory: Callable[[str], TMSF]):
        cls._text_factory = factory
        return factory

    @classmethod
    def get_text_factory(cls):
        return cls._text_factory

    @classmethod
    def register_adapter_message(
        cls, adapter: SupportedAdapters, message_class: type[Message]
    ):
        cls._message_registry[adapter] = message_class

    async def build(self, bot: Bot) -> Message:
        adapter_name = bot.adapter.get_name()
        if adapter_name not in supported_adapter_names:
            raise AdapterNotSupported(adapter_name)
        if message_type := self._message_registry.get(adapter_name):
            ms: list[MessageSegment] = await asyncio.gather(
                *[ms_factory.build(bot) for ms_factory in self]
            )
            return message_type(ms)
        raise AdapterNotInstalled(adapter_name)

    def __init__(self, message: str | Iterable[TMSF] | TMSF):
        super().__init__()

        if message is None:
            return
        elif isinstance(message, str):
            self.append(self.get_text_factory()(message))
        elif isinstance(message, MessageSegmentFactory):
            self.append(message)
        elif isinstance(message, Iterable):
            self.extend(message)

    def __add__(self: TMF, other: str | TMSF | Iterable[TMSF]) -> TMF:
        result = self.copy()
        result += other
        return result

    def __radd__(self: TMF, other: str | TMSF | Iterable[TMSF]) -> TMF:
        result = self.__class__(other)
        return result + self

    def __iadd__(self: TMF, other: str | TMSF | Iterable[TMSF]) -> TMF:
        if isinstance(other, str):
            self.append(self.get_text_factory()(other))
        elif isinstance(other, MessageSegmentFactory):
            self.append(other)
        elif isinstance(other, Iterable):
            self.extend(other)

        return self

    def append(self: TMF, obj: str | TMSF) -> TMF:
        if isinstance(obj, MessageSegmentFactory):
            super().append(obj)
        elif isinstance(obj, str):
            super().append(self.get_text_factory()(obj))

        return self

    def extend(self: TMF, obj: TMF | Iterable[TMSF]) -> TMF:
        for message_segment_factory in obj:
            self.append(message_segment_factory)

        return self

    def copy(self: TMF) -> TMF:
        return deepcopy(self)



from io import BytesIO
from pathlib import Path
from typing import TypedDict

from ..utils import MessageFactory, MessageSegmentFactory


class TextData(TypedDict):
    text: str


class Text(MessageSegmentFactory):
    """文本消息段"""

    data: TextData

    def __init__(self, text: str) -> None:
        """文本消息段

        参数:
            text: 文本内容
        """
        self.data = {"text": text}


MessageFactory.register_text_ms(lambda text: Text(text))


class ImageData(TypedDict):
    image: str | bytes | Path | BytesIO
    name: str


class Image(MessageSegmentFactory):
    """图片消息段"""

    data: ImageData

    def __init__(
        self, image: str | bytes | Path | BytesIO, name: str = "image"
    ) -> None:
        """图片消息段

        支持多种格式的数据

        参数:
            image: str 为图片 URL，bytes 为图片数据，Path 为图片路径，BytesIO 为图片文件流
            name: 图片名称，默认为 image
        """
        self.data = {"image": image, "name": name}


class MentionData(TypedDict):
    user_id: str


class Mention(MessageSegmentFactory):
    """提到其他用户"""

    data: MentionData

    def __init__(self, user_id: str):
        """提到其他用户的消息段

        参数:
            user_id: 用户 ID
        """

        self.data = {"user_id": user_id}


class ReplyData(TypedDict):
    message_id: str


class Reply(MessageSegmentFactory):
    """回复其他消息的消息段"""

    data: ReplyData

    def __init__(self, message_id: str | int):
        """回复其他消息的消息段

        参数:
            message_id: 需要回复消息的 ID
        """

        self.data = {"message_id": str(message_id)}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_saa',
 'nonebot_plugin_saa.adapters',
 'nonebot_plugin_saa.types',
 'nonebot_plugin_saa.utils']

package_data = \
{'': ['*']}

install_requires = \
['nonebot2>=2.0.0rc1,<3.0.0', 'strenum>=0.4.8,<0.5.0']

setup_kwargs = {
    'name': 'nonebot-plugin-send-anything-anywhere',
    'version': '0.1.0',
    'description': 'An adaptor for nonebot2 adaptors',
    'long_description': '<div align="center">\n\n~logo征集中，假装有图片~\n\n# Nonebot Plugin<br>Send Anything Anywhere\n\n你只管业务实现，把发送交给我们\n\n![GitHub Workflow Status](https://img.shields.io/github/actions/workflow/status/felinae98/nonebot-plugin-send-anything-anywhere/test.yml)\n![Codecov](https://img.shields.io/codecov/c/github/felinae98/nonebot-plugin-send-anything-anywhere)\n\n</div>\n\n这个插件可以做什么\n\n- 为常见的消息类型提供抽象类，自适应转换成对应 adapter 的消息\n- 提供一套统一的，符合直觉的发送接口（规划中）\n- 为复杂的消息提供易用的生成接口（规划中）\n\n本插件通过传入 bot 的类型来自适应生成对应 bot adapter 所使用的 Message\n\n## 安装\n\nTODO\n\n## 支持的 adapter\n\n- [x] OneBot v11\n- [x] OneBot v12\n\n## 支持的消息类型\n\n- [x] 文字（全平台）\n- [x] 图片（全平台）\n- [x] at（全平台)\n- [x] 回复（全平台）\n\n## 问题与例子\n\n因为在现在的 Nonebot 插件开发中，消息的构建和发送是和 adapter 高度耦合的，这导致一个插件要适配不同的 adapter 是困难的\n\nbefore:\n\n```python\nfrom nonebot.adapters.onebot.v11.event import MessageEvent as V11MessageEvent\nfrom nonebot.adapters.onebot.v11.message import MessageSegment as V11MessageSegment\nfrom nonebot.adapters.onebot.v12.event import MessageEvent as V12MessageEvent\nfrom nonebot.adapters.onebot.v12.message import MessageSegment as V12MessageSegment\nfrom nonebot.adapters.onebot.v12.bot import Bot as V12Bot\n\npic_matcher = nonebot.on_command(\'发送图片\')\n\npic_matcher.handle()\nasync def _handle_v11(event: V11MessageEvent):\n    pic_content = ...\n    msg = V11MessageSegment.image(pic_content) + V11MessageSegment.text("这是你要的图片")\n    await pic_matcher.finish(msg)\n\npic_matcher.handle()\nasync def _handle_v12(bot: V12Bot, event: V12MessageEvent):\n    pic_content = ...\n    pic_file = await bot.upload_file(type=\'data\', name=\'image\', data=pic_content)\n    msg = V12MessageSegment.image(pic_file[\'file_id\']) + V12MessageSegment.text("这是你要的图片")\n    await pic_matcher.finish(msg)\n```\n\n现在只需要:\n\n```python\nfrom nonebot.adapters.onebot.v11.event import MessageEvent as V11MessageEvent\nfrom nonebot.adapters.onebot.v12.event import MessageEvent as V12MessageEvent\nfrom nonebot.internal.adapter.bot import Bot\nfrom nonebot_plugin_saa import Image, Text, MessageFactory\n\npic_matcher = nonebot.on_command(\'发送图片\')\n\npic_matcher.handle()\nasync def _handle_v12(bot: Bot, event: Union[V12MessageEvent, V11MessageEvent]):\n    pic_content = ...\n    msg_builder = MessageFactory([\n        Image(pic_content), Text("这是你要的图片")\n    ])\n    # or msg_builder = Image(pic_content) + Text("这是你要的图片")\n    msg = await msg_builder.build(bot)\n    await pic_matcher.finish(msg)\n```\n\n## 类似项目\n\n- [nonebot-plugin-all4one](https://github.com/nonepkg/nonebot-plugin-all4one) 解决了类似的问题，但是用了不同路径\n- [nonebot-plugin-params](https://github.com/iyume/nonebot-plugin-params) 通过 Rule 定制订阅的平台，与本插件联合使用也许会有奇效\n\n## License\n\nTODO\n',
    'author': 'felinae98',
    'author_email': '731499577@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from functools import partial

from ..types import Text, Image, Reply, Mention
from ..utils import MessageFactory, SupportedAdapters, register_ms_adapter

try:
    from nonebot.adapters.onebot.v11.message import Message, MessageSegment

    adapter = SupportedAdapters.onebot_v11
    register_onebot_v11 = partial(register_ms_adapter, adapter)

    MessageFactory.register_adapter_message(SupportedAdapters.onebot_v11, Message)

    @register_onebot_v11(Text)
    def _text(t: Text) -> MessageSegment:
        return MessageSegment.text(t.data["text"])

    @register_onebot_v11(Image)
    async def _image(i: Image) -> MessageSegment:
        return MessageSegment.image(i.data["image"])

    @register_onebot_v11(Mention)
    async def _mention(m: Mention) -> MessageSegment:
        return MessageSegment.at(m.data["user_id"])

    @register_onebot_v11(Reply)
    async def _reply(r: Reply) -> MessageSegment:
        return MessageSegment.reply(int(r.data["message_id"]))

except ImportError:
    pass
except Exception as e:
    raise e
